from django.conf import settings
from django.test import RequestFactory
import htmls
from model_bakery import baker

from unittest import mock


class MockRequestResponse(object):
    """
    Return type of :meth:`.TestCaseMixin.mock_request`,
    :meth:`.TestCaseMixin.mock_http200_getrequest_htmls` and
    :meth:`.TestCaseMixin.mock_postrequest`.

    .. attribute:: response

        The HttpResponse object.

    .. attribute:: request

        The RequestFactory-generated HttpRequest object.

    .. attribute:: selector

        A ``htmls.S`` object created with the entire content of the
        response as input. Only available when using
        :meth:`.TestCaseMixin.mock_http200_getrequest_htmls`.
    """
    def __init__(self, response, request):
        self.response = response
        self.request = request
        self.selector = None


class TestCaseMixin(object):
    """
    A mixin class that makes it easier to write tests for cradmin views.

    It mocks all the required attributes of the request object.

    Examples:

        Minimalistic::

            from cradmin_legacy import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    mockresponse = self.mock_http200_getrequest_htmls()
                    mockresponse.selector.prettyprint()

                def test_post(self):
                    mockresponse = self.mock_postrequest(requestkwargs={
                        'data': {
                            'name': 'Jane Doe',
                            'age': 24
                        }
                    })
                    self.assertEqual(mockresponse.response.status_code, 302)


        Views that take arguments::

            from cradmin_legacy import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    # The kwargs for mock_postrequest, mock_http200_getrequest_htmls
                    # and mock_getrequest are the same, so only showing one for brevity.
                    mockresponse = self.mock_http200_getrequest_htmls(viewkwargs={'pk': 10})

        Views that use a querystring (GET)::

            from cradmin_legacy import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    mockresponse = self.mock_http200_getrequest_htmls(
                        requestkwargs={
                            'data': {
                                'orderby': 'name'
                            }
                        }
                    )

        Using a real user object::

            from cradmin_legacy import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_post(self):
                    requestuser = baker.make(settings.AUTH_USER_MODEL)
                    mockresponse = self.mock_http200_getrequest_htmls(requestuser=requestuser)

        Mocking Django messages framework messages::

            from cradmin_legacy import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_post(self):
                    messagesmock = mock.MagicMock()
                    mockresponse = self.mock_postrequest(data={
                        'name': 'Jane Doe',
                        'age': 24
                    }, messagesmock=messagesmock)
                    messagesmock.add.assert_called_once_with(
                        messages.SUCCESS,
                        'The data was posted successfully!',
                        '')

    """
    #: The view class - must be set in subclasses
    viewclass = None

    def create_default_user_for_mock_request(self):
        return baker.make(settings.AUTH_USER_MODEL)

    def mock_request(self, method,
                     cradmin_role=None,
                     cradmin_app=None,
                     cradmin_instance=None,
                     requestuser=None,
                     messagesmock=None,
                     sessionmock=None,
                     requestattributes={},
                     requestkwargs=None,
                     viewkwargs=None):
        """
        Create a mocked request using ``RequestFactory`` and ``mock.MagicMock``.

        Parameters:
            method: The http method (get, post, ...).
            cradmin_role: The request.cradmin_role to use. Defaults to mock.MagicMock().
            cradmin_app: The request.cradmin_app to use. Defaults to mock.MagicMock().
            cradmin_instance: The request.cradmin_instance to use. Defaults to mock.MagicMock().
            requestuser: The request.requestuser to use. Defaults to mock.MagicMock().
            sessionmock: The request.session to use. Defaults to mock.MagicMock().
            messagesmock: The request._messages to use. Defaults to mock.MagicMock().
        """
        if self.viewclass is None:
            raise NotImplementedError('You must set the viewclass attribute on TestCase classes using TestCaseMixin.')
        requestkwargs_full = {
            'path': '/'
        }
        if requestkwargs:
            requestkwargs_full.update(requestkwargs)
        viewkwargs = viewkwargs or {}
        request = getattr(RequestFactory(), method)(**requestkwargs_full)
        if requestattributes:
            for key, value in requestattributes.items():
                setattr(request, key, value)
        request.user = requestuser or self.create_default_user_for_mock_request()
        request.cradmin_role = cradmin_role or mock.MagicMock()
        request.cradmin_app = cradmin_app or mock.MagicMock()
        request.cradmin_instance = cradmin_instance or mock.MagicMock()
        request.session = sessionmock or mock.MagicMock()
        request._messages = messagesmock or mock.MagicMock()
        response = self.viewclass.as_view()(request, **viewkwargs)
        return MockRequestResponse(response=response, request=request)

    def mock_getrequest(self, **kwargs):
        return self.mock_request(method='get', **kwargs)

    def mock_http302_getrequest(self, **kwargs):
        mockresponse = self.mock_request(method='get', **kwargs)
        self.assertEqual(mockresponse.response.status_code, 302)
        return mockresponse

    def mock_http200_getrequest_htmls(self, **kwargs):
        mockresponse = self.mock_getrequest(**kwargs)
        self.assertEqual(mockresponse.response.status_code, 200)
        mockresponse.response.render()
        selector = htmls.S(mockresponse.response.content)
        mockresponse.selector = selector
        return mockresponse

    def mock_postrequest(self, **kwargs):
        return self.mock_request(method='post', **kwargs)

    def mock_http200_postrequest_htmls(self, **kwargs):
        mockresponse = self.mock_postrequest(**kwargs)
        self.assertEqual(mockresponse.response.status_code, 200)
        mockresponse.response.render()
        selector = htmls.S(mockresponse.response.content)
        mockresponse.selector = selector
        return mockresponse

    def mock_http302_postrequest(self, **kwargs):
        mockresponse = self.mock_postrequest(**kwargs)
        self.assertEqual(mockresponse.response.status_code, 302)
        return mockresponse



from __future__ import unicode_literals

from django.urls import path

from cradmin_legacy.apps.cradmin_register_account.views.begin import BeginRegisterAccountView
from cradmin_legacy.apps.cradmin_register_account.views.email_sent import EmailSentView

urlpatterns = [
    path('begin', BeginRegisterAccountView.as_view(), name="cradmin-register-account-begin"),
    path('email-sent', EmailSentView.as_view(), name="cradmin-register-account-email-sent"),
]



from django.db import models
from django.utils.translation import gettext_lazy

from cradmin_legacy.superuserui.views import mixins
from cradmin_legacy.viewhelpers import listbuilder
from cradmin_legacy.viewhelpers import listbuilderview
from cradmin_legacy.viewhelpers import listfilter
from cradmin_legacy.viewhelpers import multiselect2


class BaseView(mixins.ListFilterQuerySetForRoleMixin,
               listbuilderview.FilterListMixin,
               listbuilderview.View):
    paginate_by = 50

    def get_search_fields(self):
        """
        Get a list with the names of the fields to use while searching.

        Defaults to the ``id`` field and all CharField and TextField on the model.
        """
        fields = ['id']
        for field in self.get_model_class()._meta.get_fields():
            if isinstance(field, (models.CharField, models.TextField)):
                fields.append(field.name)
        return fields

    def add_filterlist_items(self, filterlist):
        super(BaseView, self).add_filterlist_items(filterlist=filterlist)
        search_fields = self.get_search_fields()
        if search_fields:
            filterlist.append(listfilter.django.single.textinput.Search(
                slug='search',
                label=gettext_lazy('Search'),
                label_is_screenreader_only=True,
                modelfields=search_fields))


class View(listbuilderview.ViewCreateButtonMixin,
           BaseView):
    value_renderer_class = listbuilder.itemvalue.EditDelete

    def get_filterlist_url(self, filters_string):
        return self.request.cradmin_app.reverse_appurl(
            'filter', kwargs={'filters_string': filters_string})

    def get_datetime_filter_fields(self):
        return [
            field for field in self.get_model_class()._meta.get_fields()
            if isinstance(field, models.DateTimeField)]

    def add_datetime_filters(self, filterlist):
        datetime_filter_fields = self.get_datetime_filter_fields()
        for field in datetime_filter_fields:
            filterlist.append(listfilter.django.single.select.DateTime(
                slug=field.name, label=field.verbose_name))

    def add_filterlist_items(self, filterlist):
        super(View, self).add_filterlist_items(filterlist=filterlist)
        self.add_datetime_filters(filterlist=filterlist)


class ForeignKeySelectView(BaseView):
    value_renderer_class = listbuilder.itemvalue.UseThis
    hide_menu = True

    def get_filterlist_url(self, filters_string):
        return self.request.cradmin_app.reverse_appurl(
            'foreignkeyselect-filter', kwargs={'filters_string': filters_string})


class ManyToManySelectView(multiselect2.manytomanyview.ListBuilderFilterListViewMixin,
                           BaseView):
    pass



from django.db import models

from cradmin_legacy.viewhelpers.listfilter.basefilters.single import abstracttextinput
from cradmin_legacy.viewhelpers.listfilter.django.base import DjangoOrmFilterMixin


class Search(abstracttextinput.AbstractSearch, DjangoOrmFilterMixin):
    """
    A "filter" that lets the user search the queryset.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                name = models.CharField(max_length=255)
                title = models.TextField()

        Create the filter::

            listfilter.django.single.textinput.Search(
                slug='search', label='Search',
                modelfields=['name', 'title'])
    """
    def __init__(self, *args, **kwargs):
        """
        Parameters are the same as for
        :class:`cradmin_legacy.viewhelpers.listfilter.base.abstractfilter.AbstractFilter`,
        but you can specify one extra parameter:

            - modelfields: List of model fields to search on.
        """
        self.modelfields = kwargs.pop('modelfields', None)
        super(Search, self).__init__(*args, **kwargs)

    def get_modelfields(self):
        """
        Returns a list of the model fields you want to search on.

        Defaults to the ``modelfields`` parameter for :class:`.Search`.
        """
        if self.modelfields:
            return self.modelfields
        else:
            raise NotImplementedError('You must override get_modelfields() or use the modelfields parameter.')

    def get_lookup_suffix(self):
        """
        Get the query lookup suffix. Defaults to ``"__icontains"``.
        """
        return '__icontains'

    def make_q_object_for_value(self, modelfield, cleaned_value):
        """
        Make a :class:`django.db.models.Q` object for matching the
        given ``modelfield`` with the given ``cleaned_value``.

        Args:
            modelfield (str): The modelfield to search.
            cleaned_value: The value to search for (cleaned).

        Returns:
            django.db.models.Q: A Q object.
        """
        comparison_operator = self.get_lookup_suffix()
        kwargs = {'{}{}'.format(modelfield, comparison_operator): cleaned_value}
        return models.Q(**kwargs)

    def build_query(self, modelfields, cleaned_value):
        full_query = None
        for modelfield in modelfields:
            query = self.make_q_object_for_value(modelfield=modelfield,
                                                 cleaned_value=cleaned_value)
            if full_query:
                full_query |= query
            else:
                full_query = query
        return full_query

    def filter(self, queryobject):
        modelfields = self.get_modelfields()
        cleaned_value = self.get_cleaned_value()
        if cleaned_value not in ('', None):
            queryobject = queryobject.filter(self.build_query(
                modelfields=modelfields, cleaned_value=cleaned_value))
        return queryobject


class IntSearch(abstracttextinput.IntInputFilterMixin, Search):
    """
    A "filter" that lets the user search the queryset for exact
    matches for integers.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                name = models.CharField(max_length=255)
                age = models.IntegerField()
                height = models.IntegerField()

        Create the filter::

            listfilter.django.single.textinput.IntSearch(
                slug='search_ageheight',
                label='Search for age or height',
                modelfields=['age', 'height'])

    """
    def get_lookup_suffix(self):
        return '__exact'



from cradmin_legacy import crapp
from cradmin_legacy.superuserui.views import appdashboardview


class DjangoAppCrApp(crapp.App):
    """
    Cradmin App that provides an overview of a Django app.
    """

    #: The :class:`cradmin_legacy.superuserui.superuserui_registry.DjangoAppConfig` this
    #: app belongs to.
    #: This is set automatically in
    #: :meth:`cradmin_legacy.superuserui.superuserui_registry.DjangoAppConfig.make_crapp_class`
    djangoappconfig = None

    @classmethod
    def get_appdashboardview_class(cls):
        return appdashboardview.View

    @classmethod
    def get_appurls(cls):
        return [
            crapp.Url(
                r'^$',
                cls.get_appdashboardview_class().as_view(),
                name=crapp.INDEXVIEW_NAME)
        ]



from __future__ import unicode_literals

import os
import posixpath
import uuid
from builtins import object

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy
from future.utils import python_2_unicode_compatible

from cradmin_legacy import crsettings


def archiveimage_upload_to(archiveimage, filename):
    if archiveimage.id is None:
        raise ValueError('Can not set image until after the ArchiveImage object has been created.')
    name, extension = os.path.splitext(filename)
    filenamepattern = getattr(settings, 'CRADMIN_LEGACY_IMAGEARCHIVE_FILENAMEPATTERN',
                              'cradmin_imagearchive_images/{id}-{uuid}{extension}')
    return filenamepattern.format(
        id=archiveimage.id,
        uuid=uuid.uuid1(),
        extension=extension)


class ArchiveImageManager(models.Manager):
    def filter_owned_by_role(self, role):
        return self.get_queryset().filter(
            role_object_id=role.id,
            role_content_type=ContentType.objects.get_for_model(role.__class__))


@python_2_unicode_compatible
class ArchiveImage(models.Model):
    objects = ArchiveImageManager()

    role_content_type = models.ForeignKey(
        ContentType,
        verbose_name=gettext_lazy('role'),
        help_text=gettext_lazy('The role owning this image.'),
        on_delete=models.CASCADE
    )
    role_object_id = models.PositiveIntegerField()
    role = GenericForeignKey('role_content_type', 'role_object_id')

    #: The image.
    image = models.ImageField(
        max_length=255, null=True, blank=False,
        height_field='image_height',
        width_field='image_width',
        verbose_name=gettext_lazy('image'),
        help_text=gettext_lazy('Select an image to add to the archive.'),
        upload_to=archiveimage_upload_to)

    #: The height of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_height = models.IntegerField(null=True, blank=True, editable=False)

    #: The width of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_width = models.IntegerField(null=True, blank=True, editable=False)

    #: The file extension
    file_extension = models.CharField(
        max_length=255, blank=False, null=False, editable=False)

    #: The file size in bytes.
    file_size = models.PositiveIntegerField(blank=True, null=True)

    #: The name of the image file.
    #:
    #: Can be None. This is both because this was added
    #: after cradmin was in use in production, and because
    #: calculating size can be expensive if uploading
    #: huge amounts of images in bulk. All the cradmin views
    #: provided by the app sets this.
    name = models.CharField(
        max_length=255, blank=True, null=False,
        verbose_name=gettext_lazy('name')
        # help_text=gettext_lazy('A good name helps search engines find the image, '
        #             'and it helps visually impaired users.')
    )

    #: An optional description of the image.
    description = models.TextField(
        blank=True, null=False, default='',
        verbose_name=gettext_lazy('description'),
        help_text=gettext_lazy(
            'An optional description of the image. Think if this as a description '
            'of the image for visually impaired users. This means that you should describe '
            'the information carried in the image (if any). A good description also helps '
            'search engines find the image.'),
    )

    #: Create datetime.
    created_datetime = models.DateTimeField(
        editable=False,
        auto_now_add=True
    )

    class Meta(object):
        verbose_name = gettext_lazy('archive image')
        verbose_name_plural = gettext_lazy('archive images')

    def clean(self):
        if not self.name:
            if self.image:
                self.name = os.path.splitext(posixpath.basename(self.image.name))[0]
        if self.name:
            self.file_extension = os.path.splitext(self.name)[1]

    def __str__(self):
        return self.name

    def __repr__(self):
        return 'ArchiveImage(name={name}, image_width={image_width}, image_height={image_height})'.format(
            name=self.name.encode('ascii', 'replace'),
            image_width=self.image_width,
            image_height=self.image_height)

    def get_preview_html(self, request, imagetype=None):
        """
        Get HTML that can be used to show a preview of the image.

        Parameters:
            request: A HttpRequest object which is used to create an
                absolute URI for the image.
        """
        if not imagetype:
            imagetype = crsettings.get_setting('CRADMIN_LEGACY_IMAGEARCHIVE_PREVIEW_IMAGETYPE')
        context = {
            'archiveimage': self,
            'imagetype': imagetype,
            'fallbackoptions': {
                'width': 300,
                'height': 300
            }
        }
        return render_to_string('cradmin_legacy/apps/cradmin_imagearchive/preview.django.html',
                                context, request=request)

    @property
    def screenreader_text(self):
        """
        Get screen reader text for the image. Typically added to the "alt"
        or "aria-label" HTML attributes.

        Defaults to description, but falls back to name.
        """
        if self.description:
            return self.description
        else:
            return self.name



from django.views.generic import TemplateView

from cradmin_legacy.viewhelpers import listbuilder


class ItemFrameModel(listbuilder.itemframe.Link):
    valuealias = 'modelconfig'

    def get_url(self):
        return self.modelconfig.get_indexview_url()


class ItemValueModel(listbuilder.itemvalue.FocusBox):
    valuealias = 'modelconfig'
    template_name = 'cradmin_legacy/superuserui/listbuilder/itemvalue-model.django.html'


class View(TemplateView):
    template_name = 'cradmin_legacy/superuserui/appdashboard.django.html'

    def get_models_listbuilder_list(self):
        return listbuilder.lists.RowList.from_value_iterable(
            value_iterable=self.request.cradmin_app.djangoappconfig.iter_modelconfigs(),
            frame_renderer_class=ItemFrameModel,
            value_renderer_class=ItemValueModel
        )

    def get_context_data(self, **kwargs):
        context = super(View, self).get_context_data(**kwargs)
        context['models_list'] = self.get_models_listbuilder_list()
        context['djangoappconfig'] = self.request.cradmin_app.djangoappconfig
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_temporaryfileuploadstore', '0003_temporaryfilecollection_max_filesize_bytes'),
    ]

    operations = [
        migrations.AlterField(
            model_name='temporaryfilecollection',
            name='max_filesize_bytes',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from model_bakery import baker

from cradmin_legacy.python2_compatibility import mock
import htmls
from django.test import TestCase
from django.test.client import RequestFactory
from django import http
from django import forms

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import TestModel
from cradmin_legacy.viewhelpers import multiselect

TEST_PK = 42


class DemoForm(forms.Form):
    data = forms.CharField()


class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
    form_class = DemoForm
    model = TestModel

    def get_queryset_for_role(self, role):
        return TestModel.objects.all()

    def form_valid(self, form):
        return http.HttpResponse('Submitted: {data}'.format(**form.cleaned_data))

    def get_field_layout(self):
        return ['data']


class SimpleMultiSelectView(multiselect.MultiSelectView):
    def get_queryset_for_role(self, role):
        return TestModel.objects.all()

    def object_selection_valid(self):
        return http.HttpResponse('OK')


class TestMultiSelectView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testmodelinstance = baker.make('cradmin_viewhelpers_testapp.TestModel', pk=TEST_PK)

    def test_object_selection_valid(self):
        testmodelinstance2 = baker.make('cradmin_viewhelpers_testapp.TestModel')
        testmodelinstance3 = baker.make('cradmin_viewhelpers_testapp.TestModel')

        request = self.factory.post('/test', {
            'selected_objects': [testmodelinstance2.id, testmodelinstance3.id]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"OK")

    def test_object_selection_invalid(self):
        request = self.factory.post('/test', {
            'selected_objects': [1, 10]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        selector = htmls.S(response.content)
        self.assertEqual(
            selector.one('.alert.alert-danger').text_normalized,
            'Invalid selection. This is usually caused by someone else changing permissions '
            'while you where selecting items to edit.')

    def test_object_selection_invalid_override(self):
        class SimpleMultiSelectViewOverride(SimpleMultiSelectView):
            def object_selection_invalid(self, form):
                return http.HttpResponse('Invalid selection')

        request = self.factory.post('/test', {
            'selected_objects': [1, 10]
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectViewOverride.as_view()(request)
        self.assertIn(b'Invalid selection', response.content)


class TestMultiSelectFormView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testmodelinstance = baker.make('cradmin_viewhelpers_testapp.TestModel', pk=TEST_PK)

    def test_first_load(self):

        request = self.factory.post('/test', {
            'selected_objects': [self.testmodelinstance.id],
            'is_the_multiselect_form': 'yes'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count('#cradmin_legacy_contentwrapper form'), 1)
        self.assertEqual(selector.one('input[name=selected_objects]')['type'], 'hidden')
        self.assertEqual(selector.one('input[name=selected_objects]')['value'], f'{self.testmodelinstance.id}')
        self.assertEqual(selector.count('input[name=data]'), 1)
        self.assertFalse(selector.exists('form .has-error'))

    def test_form_invalid(self):

        request = self.factory.post('/test', {
            'selected_objects': [self.testmodelinstance.id],
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count('#cradmin_legacy_contentwrapper form'), 1)
        self.assertEqual(selector.one('input[name=selected_objects]')['type'], 'hidden')
        self.assertEqual(selector.one('input[name=selected_objects]')['value'], f'{self.testmodelinstance.id}')
        self.assertEqual(selector.count('input[name=data]'), 1)
        self.assertEqual(
            selector.one('#div_id_data .help-block').alltext_normalized,
            'This field is required.')

    def test_form_valid(self):

        request = self.factory.post('/test', {
            'selected_objects': [self.testmodelinstance.id],
            'data': 'Hello world'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEqual(response.content, b'Submitted: Hello world')

    def test_form_valid_success_redirect(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = TestModel

            def get_queryset_for_role(self, role):
                return TestModel.objects.filter(id=TEST_PK)

            def get_success_url(self):
                return '/success'

            def form_valid(self, form):
                return self.success_redirect_response()

        request = self.factory.post('/test', {
            'selected_objects': [self.testmodelinstance.id],
            'data': 'Hello world'
        })
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], '/success')




#: Font-awesome icon map for the ``cradmin_icon`` template tag.
FONT_AWESOME = {
    'search': 'fa fa-search',
    'x': 'fa fa-times',
    'h1': 'fa fa-header cradmin-legacy-icon-font-lg',
    'h2': 'fa fa-header',
    'h3': 'fa fa-header cradmin-legacy-icon-font-sm',
    'bold': 'fa fa-bold',
    'italic': 'fa fa-italic',
    'link': 'fa fa-link',
    'list-ul': 'fa fa-list-ul',
    'list-ol': 'fa fa-list-ol',
    'codeblock': 'fa fa-code',
    'loadspinner': 'fa fa-spin fa-spinner',
    'caret-down': 'fa fa-caret-down',
    'caret-up': 'fa fa-caret-up',
    'pager-next-page': 'fa fa-chevron-right',
    'pager-previus-page': 'fa fa-chevron-left',
    'close-overlay-right-to-left': 'fa fa-chevron-left',
    'select-left': 'fa fa-angle-right',
    'select-right': 'fa fa-angle-left',
    'multiselect2-selected': 'fa fa-check',
}



from __future__ import unicode_literals
from builtins import range
from django.utils.html import format_html
import htmls
from cradmin_legacy.python2_compatibility import mock
from django.test import TestCase, RequestFactory
from mock_django.query import QuerySetMock

from cradmin_legacy.views.roleselect import RoleSelectView


class TestRoleSelectView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_302_redirect_on_single_role(self):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=1)
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            request = self.factory.get('/roleselecttest')
            response = RoleSelectView.as_view()(request)
            self.assertEqual(response.status_code, 302)

    def test_200_on_multiple(self):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=2)
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            request = self.factory.get('/roleselecttest')
            response = RoleSelectView.as_view()(request)
            self.assertEqual(response.status_code, 200)

    def __mock_cradmin_instance(self, roles):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=len(roles))
        cradmin_instance.get_rolequeryset.return_value.__len__.return_value = len(roles)
        cradmin_instance.get_titletext_for_role = lambda role: role.title
        cradmin_instance.get_descriptionhtml_for_role = lambda role: format_html(u'<p>{}<p>', role.description)
        cradmin_instance.get_roleid = lambda role: role.id
        cradmin_instance.rolefrontpage_url = lambda roleid: '/role/{}'.format(roleid)
        cradmin_instance.get_rolequeryset.return_value = QuerySetMock(None, *roles)
        return cradmin_instance

    def test_render_pagetitle(self):
        class CustomRoleSelectView(RoleSelectView):
            pagetitle = u'Test title'

        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = 'Role One'
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = 'Role Two'
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request = self.factory.get('/roleselecttest')
                response = CustomRoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)
                self.assertEqual(
                    selector.one('h1').alltext_normalized,
                    'Test title')

    def test_render_list_titles(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = 'Role One'
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = 'Role Two'
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request = self.factory.get('/roleselecttest')
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                self.assertEqual(selector.count('.cradmin-legacy-roleselect-role-title'), 2)
                titletextlist = [element.alltext_normalized
                                 for element in selector.list('.cradmin-legacy-roleselect-role-title')]
                self.assertEqual(titletextlist, ['Role One', 'Role Two'])

    def test_render_list_descriptions(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = 'Role One'
        role1.description = 'Role One desc'
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = 'Role Two'
        role2.description = 'Role Two desc'
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request = self.factory.get('/roleselecttest')
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                self.assertEqual(selector.count('.cradmin-legacy-roleselect-role-description'), 2)
                titletextlist = [element.alltext_normalized
                                 for element in selector.list('.cradmin-legacy-roleselect-role-description')]
                self.assertEqual(titletextlist, ['Role One desc', 'Role Two desc'])

    def test_render_list_urls(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = 'Role One'
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = 'Role Two'
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request = self.factory.get('/roleselecttest')
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                urllist = [element['href']
                           for element in selector.list('.cradmin-legacy-roleselect-list li a')]
                self.assertEqual(urllist, ['/role/1', '/role/2'])

    def test_render_pagination(self):
        class CustomRoleSelectView(RoleSelectView):
            paginate_by = 3

        roles = []
        for index in range(CustomRoleSelectView.paginate_by + 2):
            role = mock.MagicMock()
            role.id = index
            role.title = 'Role {}'.format(index)
            roles.append(role)
        cradmin_instance = self.__mock_cradmin_instance(roles=roles)

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request_page1 = self.factory.get('/roleselecttest')
                response_page1 = CustomRoleSelectView.as_view()(request_page1)
                self.assertEqual(response_page1.status_code, 200)
                response_page1.render()
                selector_page1 = htmls.S(response_page1.content)

                request_page2 = self.factory.get('/roleselecttest', {
                    'page': 2
                })
                response_page2 = CustomRoleSelectView.as_view()(request_page2)
                self.assertEqual(response_page2.status_code, 200)
                response_page2.render()
                selector_page2 = htmls.S(response_page2.content)

                self.assertEqual(selector_page1.count('.cradmin-legacy-roleselect-list li'), 3)
                self.assertEqual(selector_page2.count('.cradmin-legacy-roleselect-list li'), 2)

                self.assertTrue(selector_page1.exists('.pager-container'))
                self.assertTrue(selector_page1.exists('.pager-container li.previous.disabled'))
                self.assertFalse(selector_page1.exists('.pager-container li.next.disabled'))
                self.assertTrue(selector_page2.exists('.pager-container'))
                self.assertFalse(selector_page2.exists('.pager-container li.previous.disabled'))
                self.assertTrue(selector_page2.exists('.pager-container li.next.disabled'))

    def test_render_no_pagination(self):
        class CustomRoleSelectView(RoleSelectView):
            paginate_by = 3

        roles = []
        for index in range(CustomRoleSelectView.paginate_by):
            role = mock.MagicMock()
            role.id = index
            role.title = 'Role {}'.format(index)
            roles.append(role)
        cradmin_instance = self.__mock_cradmin_instance(roles=roles)

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch(
                'cradmin_legacy.views.roleselect.cradmin_instance_registry',
                cradmin_instance_registry):
            with mock.patch(
                    'cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry',
                    cradmin_instance_registry):
                request = self.factory.get('/roleselecttest')
                response = CustomRoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                # selector.one('#cradmin_legacy_roleselect').prettyprint()
                self.assertEqual(selector.count('.cradmin-legacy-roleselect-list li'), 3)
                self.assertFalse(selector.exists('.pager-container'))



from __future__ import unicode_literals

from django.urls import path

from cradmin_legacy.apps.cradmin_authenticate.views.login import LoginView
from cradmin_legacy.apps.cradmin_authenticate.views.logout import cradmin_logoutview

urlpatterns = [
    path('login', LoginView.as_view(), name='cradmin-authenticate-login'),
    path('logout', cradmin_logoutview, name='cradmin-authenticate-logout'),
]



import json
import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'cradmin_legacy', 'version.json')) as f:
    version = json.loads(f.read())

setup(
    name='cradmin-legacy',
    description='Legacy 1.x fork of django cradmin.',
    version=version,
    url='https://github.com/appressoas/cradmin_legacy',
    author='Espen Angell Kristiansen, Tor Johansen, Vegard Angell, Magne Westlie',
    author_email='post@appresso.no',
    license='BSD',
    packages=find_packages(
        exclude=[
            'ez_setup',
            'tasks'
        ]),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'setuptools',
        'Django>=3.2.0,<4',
        'django-cradmin>=9.0.0,<10.0.0',
        'ievv-opensource>=8.0.0,<9.0.0',
        'django-multiupload',
        'Jinja2',
        'pytz',
        'future',
        'html2text'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
    ]
)



from __future__ import unicode_literals
from django.test import TestCase
from cradmin_legacy.utils import crhumanize


class TestHumanReadableFileSize(TestCase):
    def test_less_than_kb(self):
        self.assertEqual('0B', crhumanize.human_readable_filesize(0))
        self.assertEqual('999B', crhumanize.human_readable_filesize(999))

    def test_kb(self):
        self.assertEqual('1KB', crhumanize.human_readable_filesize(1000))
        self.assertEqual('999KB', crhumanize.human_readable_filesize(999999))

    def test_mb(self):
        self.assertEqual('1.0MB', crhumanize.human_readable_filesize(1000000))
        self.assertEqual('55.6MB', crhumanize.human_readable_filesize(55555555))

    def test_gb(self):
        self.assertEqual('1.00GB', crhumanize.human_readable_filesize(1000000000))
        self.assertEqual('55.56GB', crhumanize.human_readable_filesize(55555555555))

    def test_tb(self):
        self.assertEqual('1.00TB', crhumanize.human_readable_filesize(1000000000000))
        self.assertEqual('55.56TB', crhumanize.human_readable_filesize(55555555555555))


class TestDehumanizeReadableFileSize(TestCase):
    def test_less_than_kb(self):
        self.assertEqual(0, crhumanize.dehumanize_readable_filesize('0B'))
        self.assertEqual(999, crhumanize.dehumanize_readable_filesize('999B'))

    def test_kb(self):
        self.assertEqual(1000, crhumanize.dehumanize_readable_filesize('1KB'))
        self.assertEqual(999000, crhumanize.dehumanize_readable_filesize('999KB'))

    def test_mb(self):
        self.assertEqual(1000000, crhumanize.dehumanize_readable_filesize('1.0MB'))
        self.assertEqual(55600000, crhumanize.dehumanize_readable_filesize('55.6MB'))

    def test_gb(self):
        self.assertEqual(1000000000, crhumanize.dehumanize_readable_filesize('1.00GB'))
        self.assertEqual(55560000000, crhumanize.dehumanize_readable_filesize('55.56GB'))

    def test_tb(self):
        self.assertEqual(1000000000000, crhumanize.dehumanize_readable_filesize('1.00TB'))
        self.assertEqual(55560000000000, crhumanize.dehumanize_readable_filesize('55.56TB'))



from __future__ import unicode_literals

from django.test import TestCase
from future import standard_library
from model_bakery import baker

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import FilterTestModel
from cradmin_legacy.viewhelpers import listfilter

standard_library.install_aliases()


class TestSearch(TestCase):
    def test_no_value(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug='mycharfield',
            modelfields=['mycharfield', 'mytextfield'])
        testfilter.set_values(values=[])
        withvalue1 = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mycharfield='A testvalue')
        withvalue2 = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mytextfield='Another testvalue')
        emptyvalue = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mycharfield='')
        nullvalue = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                               mycharfield=None)
        self.assertEqual(
            {withvalue1, withvalue2, emptyvalue, nullvalue},
            set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_matching_single(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug='mycharfield',
            modelfields=['mycharfield', 'mytextfield'])
        testfilter.set_values(values=['Another'])
        baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='A testvalue')
        withvalue2 = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mytextfield='Another testvalue')
        baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='')
        baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield=None)
        self.assertEqual(
            {withvalue2},
            set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_matching_multiple(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug='mycharfield',
            modelfields=['mycharfield', 'mytextfield'])
        testfilter.set_values(values=['testvalue'])
        withvalue1 = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mycharfield='A testvalue')
        withvalue2 = baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                                mytextfield='Another testvalue')
        baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='')
        baker.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield=None)
        self.assertEqual(
            {withvalue1, withvalue2},
            set(testfilter.filter(queryobject=FilterTestModel.objects.all())))



from __future__ import unicode_literals
from django.db import models
from django.utils import timezone


class TestModel(models.Model):
    testfield = models.CharField(verbose_name=u'Test Value', max_length=100)


class FilterTestModel(models.Model):
    mybooleanfield = models.BooleanField(default=False)
    mycharfield = models.CharField(null=True, blank=True, default='', max_length=255)
    mytextfield = models.TextField(null=False, blank=False, default='')
    myintnullfield = models.IntegerField(default=None, null=True, blank=True)
    mydatetimefield = models.DateTimeField(default=timezone.now)
    mynulldatetimefield = models.DateTimeField(null=True, blank=True, default=None)

    def __str__(self):
        return 'FilterTestModel(mycharfield={!r}, mybooleanfield={})'.format(
            self.mycharfield, self.mybooleanfield)



import time

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin


class DelayMiddleware(MiddlewareMixin):
    """
    To use this, you must add the following to your settings:

    - Add ``cradmin_legacy.delay_middleware.DelayMiddleware`` to ``MIDDLEWARE``.
    - Set ``CRADMIN_LEGACY_DELAY_MIDDLEWARE_MILLISECONDS`` to the number of milliseconds
      delay you want to add to all requests (I.E.: 2000 for 2 seconds).
    """
    def process_request(self, request):
        time.sleep(settings.CRADMIN_LEGACY_DELAY_MIDDLEWARE_MILLISECONDS / 1000.0)
        return None
